#pragma once
#include "main.h"

#define VX(v)     ((v).x)
#define VY(v)     ((v).y)
#define VMOD(v)   ((v).mod)
#define VANGLE(v) ((v).angle)

#define MOD(x,y)  (sqrt((x)*(x) + (y)*(y)))
#define ANGLE(x,y)(((x)==0.) ? 0. : (((x)<0.)?atan((y)/(x))+M_PI:atan((y)/(x))))

#define DIST(v,u) MOD((v).x-(u).x, (v).y-(u).y)

// Misc
double angle_diff(const double ref, double a);

// Base of 2D vectors.
typedef struct {
  double x, y; // Cartesian values.
  double mod, angle; // Polar values.
} Vec2;

// Vector manupulation.
void vect_cset(Vec2* v, const double x, const double y);
// Doesn't set mod nor angle.
void vect_csetmin(Vec2* v, const double x, const double y);
void vect_pset(Vec2* v, const double mod, const double angle);
void vectcpy(Vec2* dest, const Vec2* src);
void vectnull(Vec2* v);
double vect_angle(const Vec2* ref, const Vec2* v);

// Describe any solid in 2D space.
struct Solid {
  double mass, dir, dir_vel; // Properties.
  Vec2 vel, pos, force; // Position/velocity vectors.
  void(*update)(struct Solid*, const double); // Update method.
};

typedef struct Solid Solid;

// Solid manipulation.
void solid_init(Solid* dest, const double mass, const double dir, 
      const Vec2* pos, const Vec2* vel);
Solid* solid_create(const double mass, const double dir, 
      const Vec2* pos, const Vec2* vel);
void solid_free(Solid* src);

