#pragma once
#include "main.h"
#include "physics.h"
#include "ai.h"
#include "outfit.h"
#include "ship.h"

// Aproximation for pilot size.
#define PILOT_SIZE_APROX  0.8
#define PILOT_DISABLED    0.2 // Based on armor percentage.

// Creation flags.
#define PILOT_PLAYER    1 // Pilot is a player.

typedef struct {
  Outfit* outfit; // Associated outfit.
  unsigned int quantity; // Number of outfits of this type that the pilot has.
  unsigned int timer; // Used to store last used weapon time.
} PilotOutfit;

// Primary pilot structure.
typedef struct Pilot {
  unsigned int id; // Pilots id.
  char* name;     // Pilot's name (if unique).

  // Object characteristics.
  Ship* ship;     // Pilots ship.
  Solid* solid;   // Associated solid (physics).

  // Current health.
  double armor, shield, energy;
  double armor_max, shield_max, energy_max;

  void (*think)(struct Pilot*); // AI thinking for the pilot.
  void (*update)(struct Pilot*, const double); // Update the pilot.
  void (*render)(struct Pilot*); // Rendering the pilot.

  // Outfit management.
  PilotOutfit* outfits;

  unsigned int properties; // Used for AI etc.

  // AI.
  Task* task; // Current action.
} Pilot;

// Grabing pilot crap.
extern Pilot* player; // The player.
Pilot* get_pilot(unsigned int id);
unsigned int pilot_getNext(unsigned int id);

// MISC.
void pilot_shoot(Pilot* p, int secondary);
void pilot_hit(Pilot* p, double damage_shield, double damage_armor);

// Creation.
void pilot_init(Pilot* dest, Ship* ship, char* name, const double dir,
      const Vec2* pos, const Vec2* vel, const int flags);

unsigned int pilot_create(Ship* ship, char* name, const double dir,
      const Vec2* pos, const Vec2* vel, const int flags);

// Cleanup.
void pilot_destroy(Pilot* p);
void pilots_free(void);

// Update.
void pilots_update(double dt);

